/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1.dwr;

import net.opengis.gml.v_3_1_1.LineStringPropertyType;
import net.opengis.gml.v_3_1_1.LineStringType;
import net.opengis.gml.v_3_1_1.MultiLineStringType;

import org.directwebremoting.extend.MarshallException;
import org.jvnet.jaxb2_commons.locator.DefaultRootObjectLocator;
import org.jvnet.ogc.gml.v_3_1_1.jts.ConversionFailedException;

import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;

public class MultiLineStringTypeConverter
    extends
    AbstractGeometryTypeConverter<MultiLineStringType, double[][][]> {
  private LineStringTypeConverter lineStringTypeConverter = new LineStringTypeConverter();

  @Override
  protected Class<double[][][]> getCoordinatesType() {
    return double[][][].class;
  }

  @Override
  protected double[][][] createCoordinates(MultiLineStringType multiLineStringType)
      throws MarshallException {

    try {
      final MultiLineString multiLineString = (MultiLineString) getConverter().createGeometry(
          new DefaultRootObjectLocator(multiLineStringType),
          multiLineStringType);

      final int numGeometries = multiLineString.getNumGeometries();
      double[][][] coordinates = new double[numGeometries][][];
      for (int index = 0; index < numGeometries; index++) {
        coordinates[index] = lineStringTypeConverter.createCoordinates((LineString) multiLineString
            .getGeometryN(index));
      }
      return coordinates;
    }
    catch (ConversionFailedException cfex) {
      throw new MarshallException(LineStringType.class, cfex);
    }

  }

  @Override
  protected String getGeometryType() {
    return "MultiLineString"; //$NON-NLS-1$
  }

  @Override
  protected MultiLineStringType createGeometry(double[][][] coordinates) {
    final MultiLineStringType multiLineString = new MultiLineStringType();

    for (int index = 0; index < coordinates.length; index++) {
      final double[][] lineStringCoordinates = coordinates[index];

      LineStringType lineStringType = lineStringTypeConverter.createGeometry(lineStringCoordinates);
      LineStringPropertyType lineStringPropertyType = new LineStringPropertyType();
      lineStringPropertyType.setLineString(lineStringType);

      multiLineString.getLineStringMember().add(lineStringPropertyType);

    }

    return multiLineString;
  }
}
