/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1.dwr;

import net.opengis.gml.v_3_1_1.MultiPointType;
import net.opengis.gml.v_3_1_1.PointPropertyType;
import net.opengis.gml.v_3_1_1.PointType;

import org.directwebremoting.extend.MarshallException;
import org.jvnet.jaxb2_commons.locator.DefaultRootObjectLocator;
import org.jvnet.ogc.gml.v_3_1_1.jts.ConversionFailedException;

import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.Point;

public class MultiPointTypeConverter
    extends
    AbstractGeometryTypeConverter<MultiPointType, double[][]> {
  private PointTypeConverter pointTypeConverter = new PointTypeConverter();

  @Override
  protected double[][] createCoordinates(MultiPointType multiPointType) throws MarshallException {
    try {
      final MultiPoint multiPoint = (MultiPoint) getConverter().createGeometry(
          new DefaultRootObjectLocator(multiPointType),
          multiPointType);
      int numGeometries = multiPoint.getNumGeometries();
      double[][] coordinates = new double[numGeometries][];
      for (int i = 0; i < numGeometries; i++) {
        coordinates[i] = pointTypeConverter.createCoordinate((Point) multiPoint.getGeometryN(i));
      }
      return coordinates;
    }
    catch (ConversionFailedException cfex) {
      throw new MarshallException(PointType.class, cfex);
    }
  }

  @Override
  protected String getGeometryType() {
    return "MultiPoint";
  }

  @Override
  protected MultiPointType createGeometry(double[][] coordinates) {
    final MultiPointType multiPoint = new MultiPointType();

    for (int index = 0; index < coordinates.length; index++) {
      final double[] pointCoordinates = coordinates[index];

      PointType pointType = pointTypeConverter.createGeometry(pointCoordinates);
      PointPropertyType pointPropertyType = new PointPropertyType();
      pointPropertyType.setPoint(pointType);

      multiPoint.getPointMember().add(pointPropertyType);

    }

    return multiPoint;
  }

  @Override
  protected Class<double[][]> getCoordinatesType() {
    return double[][].class;
  }

}
