/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wpspd.v_1_0_0.sample;

import java.util.Arrays;
import java.util.Collection;

import javax.xml.bind.JAXBElement;

import net.disy.ogc.wps.v_1_0_0.annotation.InputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.OutputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Process;
import net.disy.ogc.wps.v_1_0_0.annotation.ProcessMethod;
import net.disy.ogc.wpspd.v_1_0_0.Geometry;
import net.disy.ogc.wpspd.v_1_0_0.GeometryType;
import net.disy.ogc.wpspd.v_1_0_0.Group;
import net.disy.ogc.wpspd.v_1_0_0.GroupType;
import net.disy.ogc.wpspd.v_1_0_0.Marker;
import net.disy.ogc.wpspd.v_1_0_0.MarkerType;
import net.disy.ogc.wpspd.v_1_0_0.Message;
import net.disy.ogc.wpspd.v_1_0_0.MessageType;
import net.disy.ogc.wpspd.v_1_0_0.WpspdUtils;
import net.opengis.gml.v_3_1_1.AbstractGeometryType;

import org.jvnet.ogc.gml.v_3_1_1.jts.JTSToGML311GeometryConverter;

import com.vividsolutions.jts.geom.Polygon;

@Process(id = "sampleGroupOfGeometricMarkersWithMessagesProcess", title = "Sample group of geometric markers process", description = "Returns a group of geometric markers with messages")
public class SampleGroupOfGeometricMarkersWithMessagesAnnotatedObject {

  private final net.disy.ogc.wpspd.v_1_0_0.ObjectFactory wpspdObjectFactory = WpspdUtils
      .createObjectFactory();
  private final JTSToGML311GeometryConverter geometryConverter = new JTSToGML311GeometryConverter();

  @ProcessMethod
  @OutputParameter(id = "markers", title = "Markers", description = "Resulting geometric markers")
  public Group execute(
      @InputParameter(id = "firstGeometry", title = "First geometry", description = "First geometry") Polygon one,
      @InputParameter(id = "secondGeometry", title = "Second geometry", description = "Second geometry") Polygon two,
      @InputParameter(id = "thridGeometry", title = "Third geometry", description = "Third geometry", optional = true) Polygon three) {

    Collection<Polygon> polygons = Arrays.asList(one, two, three);

    GroupType groupType = wpspdObjectFactory.createGroupType();
    Group group = wpspdObjectFactory.createGroup(groupType);

    for (Polygon polygon : polygons) {
      if (polygon != null) {
        groupType.getPresentationDirective().add(createMarker(polygon));
      }
    }
    return group;
  }

  private Marker createMarker(Polygon polygon) {
    final MarkerType markerType = wpspdObjectFactory.createMarkerType();
    markerType.setGeometry(createGeometry(polygon));
    markerType.setMessage(createMessage(polygon));
    final Marker marker = wpspdObjectFactory.createMarker(markerType);
    return marker;
  }

  private Geometry createGeometry(Polygon polygon) {
    @SuppressWarnings("unchecked")
    final JAXBElement<AbstractGeometryType> geometryElement = (JAXBElement<AbstractGeometryType>) geometryConverter
        .createElement(polygon);

    final GeometryType geometryType = wpspdObjectFactory.createGeometryType();
    geometryType.setGeometry(geometryElement);

    final Geometry geometry = wpspdObjectFactory.createGeometry(geometryType);
    return geometry;
  }

  private Message createMessage(Polygon polygon) {
    final MessageType messageType = wpspdObjectFactory.createMessageType();
    messageType.setContent("Area:" + polygon.getArea());
    final Message message = wpspdObjectFactory.createMessage(messageType);
    return message;
  }

}
